function handle = barplotcol(H,ColorOption,maxRowCol)
% handle = barplotcol(H,ColorOption,maxRowCol)
% handle = barplotcol(H)
%
% BARPLOTCOL visualizes a data matrix as a "3D city block". The basic idea is
% the routine bar3.m, but the color is refering to the height information 
% instead of the column of the matrix. Inf and NaN elmenents are replaced by 
% zero before visualization.
%
% IN:
% H: ............ 2-dimensional data matrix                                  [-][NxM]
% ColorOption:... choice between                                                ['string']
%                  * 'floor': colored layers indicated the same height 
%                  * 'building': color indicates the value in the marix
% maxRowCol:..... option to increase the maximal number of columns and rows  [-][1x1]
%                 default: no plot for more than 10000 elements
%
% OUT:
% handle:........ handle of the graphics for further improvement
%
% See also: --
%
% sub-routines:
%    *   --
%
% REMARKS:
%    *   The size of the matrix is limited by the bar3.m function.
%    *   For more than 10000 elements, the bar-routine is very slow and you
%        get an warning now. If you really want more, increase the limit by 
%        the additional input of 'maxRowCol'.
% 
% EXAMPLE:
%        H = magic(10);
%        H = [H(1:5,:), -H(6:10,:)];
%        H(3,5) = NaN; H(3,7) = Inf;
%        H(5,1) = -Inf;H(1,9) = 0;
%        figure
%        subplot(231);barplotcol(H,'floor'); title('ColorOption: floor')
%        subplot(232);barplotcol(H','floor'); title('ColorOption: floor')
%        subplot(234);barplotcol(H,'building'); title('ColorOption: building')
%        subplot(235);barplotcol(H','building'); title('ColorOption: building')
%        subplot(2,3,[3,6]); bar3(H); title('bar3.m')

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Markus ANTONI (MA), GI, Uni Stuttgart
% -------------------------------------------------------------------------
% revision history:
%    2021-05-04: MA, unified help text in VISBUNDLE
%    2013-02-20: MA, renamed for visBundle 
%    2013-02-12: MR, prushed up of help text 
%    2010-10-21: MA, initial version at DEOS
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% -------------------------------------------------------------------------


%% checking the input arguments
narginchk(1,3)
if nargin<2
    ColorOption = 'building';
end
if nargin<3
    maxRowCol = 10000;
end

% number of elements
[row,column] = size(H);
if row*column>maxRowCol
    warning('amount of elements is quite large for bar3, increase >> maxRowCol << if you really want this')
    return
end

%% removing NaN/Inf
index    = isnan(H);
H(index) = 0;
index    = isinf(H);
H(index) = 0;



% basic is the function BAR3
handle = bar3(H,'detached');
row6   = row*6;
set(handle,'FaceAlpha',1);
set(gcf, 'Renderer', 'ZBuffer');


switch ColorOption
case 'building'
    % constant color per block
    for n=1:column
        index = logical(kron(H(:,n) == 0,ones(6,1)));
        colorData = get(handle(n),'ZData');
        colorData(index,:) = nan;
        % changing the color of all 6 faces per block
        for k = 1:6:row6;
            colordata1 = colorData(k:k+5,:);
            index1     = colordata1 == 0;
            colordata1(index1) = colordata1(2,2);

            colorData(k:k+5,:) = colordata1;
        end
        set(handle(n),'cData',colorData);
    end    
case 'floor'
    % equal color for equal height layer
    for n=1:column
        index     = logical(kron(H(:,n) == 0,ones(6,1)));
        colorData = get(handle(n),'ZData');
        colorData(index,:) = nan;
        set(handle(n),'cData',colorData);
    end
otherwise
    warning('chosen ''ColorOption'' is not implemented')
end


shading interp; axis tight; 
colorbar

