function [lam,phi,system180] = circ4sph(lam0, phi0, beta, lam_limit, phi_limit, omega0, max_dist)
% [lam,phi,system180] = circ4sph(lam0, phi0, beta, omega0, lam_limit, phi_limit)
%
% CIRC4SPH calculates circles on the sphere w.r.t. an arbitrary reference point 
% with the coordinates (lam0,phi0). If the circles pass the mapping border, then 
% NaNs are inserted. Possible applications are the visualization of spherical caps
% around any location, or a point-dependent coordinate system of spherical distances
% and azimuths in the Stokes' approach. 
% 
% IN:
% lam0: ......... longitude of reference point                             [deg][1x1]
% phi0: ......... latitude of reference point                              [deg][1x1]
% beta: ......... spherical distance/azimuth w.r.t. to reference point     [deg][nx1]
% lam_limit: .... limits of the map in longitude direction                 [deg][2x1]
%                  <default: lam_limit = [-180,+180]>
% phi_limit: .... limits of the map in latitude direction                  [deg][2x1]
%                  <default: phi_limit = [-90,+90]>
% ---- only for azimuth lines --------------------------------------------------
% omega0: ....... rotation angle of the azimuth lines                     [deg][1x1] 
% max_dist: ..... cut-off the azimuth lines at a distance                 [deg][1x1] 
%
% OUT:
% lam: .......... longitude coordinates of the circles/lines               [deg][Nx1]
% phi: .......... latitude coordinates of the circles/lines                [deg][Nx1]
% sphere180: information about the detected longitude system              [bool][1x1]
%                  * true:  use of/switch to [-180/180]-longitude system
%                  * false: use of/switch to [0,360]-longitude system
%
% See also: --
%
% sub-routines:
%    UBERALL/R2  
%    UBERALL/R3 
%    UBERALL/jump2nan
% 
% EXAMPLE:
%        figure
%        h = subplot(221); 
%        lam_limit = [-180,180]; phi_limit = [-90,90]; % <default>
%        [lam,phi] = circ4sph(-30, 60, [15:15:180]);
%        [lamAz,phiAz] = circ4sph(-30, 60, [30:30:360],lam_limit,phi_limit,0);
%        plot(lam,phi,'k'); hold on; plot(lamAz,phiAz,'r');xlim(lam_limit);ylim(phi_limit)
%        xlabel('longitude in [deg]'); ylabel('latitude in [deg]')
%        set(h,'xtick',[-180:30:180]); set(h,'ytick',[-90:30:90])
%        title('circles around (-30/60) in a cylindrical map [-180,180] x [-90,90]')
%
%        subplot(222)
%        [x,y,z] = sphere(20);
%        plot3(cosd(lam).*cosd(phi),sind(lam).*cosd(phi),sind(phi),'k');
%        hold on; surf(x,y,z,'FaceColor',0.8*[1,1,1]); axis equal; axis off
%        plot3(cosd(lamAz).*cosd(phiAz),sind(lamAz).*cosd(phiAz),sind(phiAz),'r');
%        title('circles around (-30/60) on the sphere')
%
%        h = subplot(223); 
%        lam_limit = [0,360]; phi_limit = [-90,90];
%        [lam,phi,system180] = circ4sph(40, 40, [15:15:90],lam_limit,phi_limit);
%        [lamAz,phiAz] = circ4sph(40, 40, [0:30:360],lam_limit,phi_limit,0,90);
%        plot(lam,phi,'k'); hold on; plot(lamAz,phiAz,'r');xlim(lam_limit);ylim(phi_limit)
%        title('circles around (40/40) in a cylindrical map [0,360] x [-90,90]');
%        xlabel('longitude in [deg]'); ylabel('latitude in [deg]')
%        set(h,'xtick',[0:30:360]); set(h,'ytick',[-90:30:90])
%
%       h = subplot(224); 
%       [lam,phi] = circ4sph(330, -30, [0:10:60],lam_limit,[-60,60]);
%       [lamAz,phiAz] = circ4sph(330, -30, [0:30:360],lam_limit,[-60,60],0,60);
%       plot(lam,phi,'k'); hold on; plot(lamAz,phiAz,'r');xlim(lam_limit);ylim(phi_limit)
%       title('circles around (330/-30) in a cylindrical map + cutoff option')  
%       set(h,'xtick',[0:30:360]); set(h,'ytick',[-90:30:90])
%

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Markus ANTONI (MA), GI, Uni Stuttgart
% -------------------------------------------------------------------------
% revision history:
%    2021-05-04: MA, unified help text in VISBUNDLE
%    2021-04-07: MA, initial version
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% -------------------------------------------------------------------------


%% default values
narginchk(3,7)
if nargin < 4,    lam_limit = [-180,180];  end
if nargin < 5,    phi_limit = [-90,90];    end
if nargin < 6,    omega0 = [];             end
if nargin < 7,    max_dist = [];           end
  
%% checking of dimensions
if numel(lam0)~=1 ||   numel(phi0)~=1
    error('Please insert only ONE reference point P = (lam0,phi0)')
end
beta =  unique(beta(:))';
if max(beta) < 6
    warning('Values are small; are the spherical distance/azimuth values really given in degree?')
end
beta(beta< 0)   = [];
switch numel(omega0) 
case 1
    %% lines through North pole with fixed azimuth values 
    beta(beta> 360) = [];
    azimuth = beta;
    if isscalar(max_dist)  == true
        if max_dist > 180
          max_dist = 180; 
        end
        if max_dist < 0
          max_dist = 180; 
        end
        beta = [0:.2:max_dist, NaN]';
    else
        beta = [0:.2:180, NaN]';
    end
    % meridians through the pole  
    X = bsxfun(@times, sind(beta),cosd(azimuth));
    Y = bsxfun(@times, sind(beta), sind(azimuth));
    Z = bsxfun(@times, cosd(beta) , azimuth*0+1);
    
otherwise
    %% circles around point North pole with fixed spherical distances '
    omega0 = 0;
    beta(beta> 180) = [];
    
    % parallel cicles around the pole  
    azimuth = [0:.2:360, NaN]';
    
    X = bsxfun(@times, cosd(azimuth), sind(beta));
    Y = bsxfun(@times, sind(azimuth), sind(beta));
    Z = bsxfun(@times, azimuth*0+1, cosd(beta));
end




% rotation to new location
xyz(3,:) = Z(:)';  
xyz(2,:) = Y(:)';  
xyz(1,:) = X(:)';    
try 
    R = R3(-lam0*pi/180,'3x3')*R2(-(pi/2-phi0*pi/180),'3x3') * R3(-omega0*pi/180,'3x3');
catch
    error('Please add the UBERALL-bundle to MATLAB''s search path')
end
xyz = R * xyz;
[lam,phi] = cart2sph(xyz(1,:),xyz(2,:),xyz(3,:));
lam = lam*180/pi;
phi = phi*180/pi;


% assuming the [-180,180]-longitude system 
system180 = true; 


if numel(lam_limit) == 2
    maxLam = max(lam_limit);
    minLam = min(lam_limit);

    if minLam >= 0
       % assuming the [0,360]-longitude system 
       neg = lam<0;
       lam(neg) = 360+lam(neg);      
       system180 = false;
    end
    [lam,phi] = jump2nan(lam, 0.5*(maxLam-minLam),phi);

    % removing elements outside the region of interest
    out =  lam>maxLam; 
    phi(out) = NaN;
    lam(out) = NaN;
    
    out = lam<minLam;
    phi(out) = NaN;
    lam(out) = NaN;
    
else
    warning('please enter the longitude boundary ''lam_limit'' as a [2x1] vector')
end

if numel(phi_limit) == 2
    maxPhi = max(phi_limit);
    minPhi = min(phi_limit);
    
    % removing elements outside the region of interest
    out =  phi>maxPhi-1; 
    phi(out) = NaN;
    lam(out) = NaN;
    
    out = phi<minPhi+1;
    phi(out) = NaN;
    lam(out) = NaN;
else
    warning('please enter the latitude boundary ''phi_limit'' as a[2x1] vector')
end


  