function h = graypatch(xStartEnd, Ylim, Xtick,white1)
% h = graypatch(xStartEnd, Ylim, Xtick,white1)
%
% GRAYPATCH hightlights 'periodic' events by marking every second period 
% with an alternating white/gray backgroud. Graypatch can also deal with 
% non-equal time spans (see example).
% 
% IN:
%   xStartEnd .. defines the time points of the events, 
%                [start, period_increment, end] for regular spacing             [1x3]
%                [T0, T1; T2, T3; ...] for irregular spacing                    [Nx2]
%   Ylim ....... limits of y-axis                                               [1x2]
%   Xtick ...... labels on the x-axis (e.g. periods T*n)                        [1xN]
%   white1...... first patch is white for white1 = true (default) or            [1x1]
%                gray for white1 = false         
%                 
% OUT:
%  h:............ handle of the graphics for further improvement
%
% REMARKS:
%    *   To avoid "hidden yticks", is recommanded to start with a white patch
%        which is the default in the new version
%
% EXAMPLE:
%        figure;
%        subplot(211);
%        t = -5:.01:20;
%        f1 = sin(t*pi);
%        plot(t, f1); hold on;
%        graypatch([-5,2,20], [-1.2,1.2], [-5:1:20],true);
%        title('graypatch(..) with regular intervals');
% 
%        subplot(212); 
%        f2 = sin(sign(t).*sqrt(abs(t))*pi).*exp(-t/10);
%        plot(t,f2); hold on;
%        xStartEnd = [-5,-4;0,4; 16,20];
%        graypatch(xStartEnd,[-3,3],[-5 -4 0 4 16 20]);
%        title('graypatch(..) with irregular intervals');

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Markus ANTONI (MA), GI, Uni Stuttgart
%    Matthias ROTH (MR), GI, Uni Stuttgart
% -------------------------------------------------------------------------
% revision history:
%    2021-05-04: MA, start with white patch to avoid hidden yticks
%    2016-03-14: MR, brush-up help text; remove alpha bug: plotted gray
%                    bars on top of the data, hence changing their color;
%                    change code, that also fractions of periods are
%                    possible
%    2015-03-17: MA, some modifications
%    2015-02-19: MA, initial version, based on gray_patch of 
%                    Mohammad J. TOURIAN
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% ------------------------------------------------------------------------- 

%% checking the input arguments
narginchk(2,4)
if nargin < 4,   white1 = true;   end
if nargin < 3,   Xtick = [];      end

if ~isscalar(white1), error('argument ''white1'' must be scalar'); end
if ~islogical(white1), error('argument ''white1'' must be boolian'); end
if isscalar(Xtick) && islogical(Xtick), white1 = Xtick; end
    
%% create Start-End matrix from [start, period_increment, end]
if numel(xStartEnd) == 3
    V(1, :) = xStartEnd(1)+white1*xStartEnd(2):(2*xStartEnd(2)):xStartEnd(3);
    V(2, :) = V(1, :) + xStartEnd(2);
    V(V > xStartEnd(3)) = xStartEnd(3);
else % we expect non-periodic elements
    V      = xStartEnd';
    
end
blocks = numel(V(1, :));
if blocks < 2
    warning('graypatch requires more than 1 patch');
    return
end


%% prepate patch coordinates 
X = V([1; 1; 2; 2], 1:blocks);
Y = repmat(Ylim([1, 2, 2, 1])', 1, blocks);
Z = ones(size(X));

%% gray blocks
h = patch(X, Y, Z, 'facecolor', [0.94 0.94 0.94], 'LineStyle', ':'); hold on;
try 
    uistack(h, 'bottom');     % put gray blocks on lowest graphics layer
catch
    warning('''uistack''-routine is missing, gray blocks are not in lowest layer now')
end
grid off; 

%% labeling, grid
Xlim = [min(V(:)),max(V(:))];
if nargin > 2
    if isvector(Xtick) == 1 && numel(Xtick) > 1
        set(gca, 'xtick', unique(Xtick))
        Xlim = [min(Xtick), max(Xtick)];
        xlim(Xlim);
    else
        if islogical(Xtick) == false
            error('''Xtick'' must be given as a vector in increasing order')
        end
    end    
end

%% add x- and y-axis 
if prod(Ylim) < 0
    plot(Xlim, [0,0], 'k', 'LineWidth', 2)
end
if prod(Xlim) < 0
    plot([0, 0], Ylim, 'k', 'LineWidth', 2)
end
ylim(Ylim);
grid on;

try 
    % improvement of axes, maybe only for new versions of MATLAB?
    ax = gca;
    ax.Layer = 'top';
catch
    warning('ax.Layer = ''top'' does not work in your enviroment, axes are not in top layer now')
end
