function handle = plotcol(x,y,z,c,linestyle)
% handle = plotcol(x,y,z,c,linestyle)
% 
% PLOTCOL visualizes observation locations – with 1D, 2D or 3D coordinates – 
% and the observation values, which define then the colors. 
% 
% PLOTCOL(X,Y,Z,C,LINESTYLE) plots a colored parametric line based on 
% X, Y, Z, and C using the line style LINESTYLE.  The color scaling is
% determined by the values of C or by the current setting of CAXIS.  The
% scaled color values are used as indices into the current COLORMAP.
%
% Any combination of inputs can be used.  If C is not
% given, it is assigned to Z, Y, or X, depending on the
% input.  Below is a table which describes this:
%
%   GIVEN       VALUE OF C
%   X,Y,Z           Z
%    X,Y            Y
%     X             X
%
% IN:
% x ............. x-coordinates of the observations                             [NxM]
% y ............. y-coordinates of the observations                             [NxM] 
% z ............. z-coordinates of the observations                             [NxM] 
% c ............. value of the observations                                     [NxM] 
% linestyle:      defining the line/marker in MATLAB notation ('.ox+*sdv^<>ph')
% 
% OUT:
% handle:........ handle of the graphics for further improvement 
%
% See also:  PLOTCOLM
%
% sub-routines:
%    *   --
% 
% EXAMPLE:
%        r = 1:.2:20;
%        x = r.*cos(r);        y = r.*sin(r);
%        z = sqrt(x.^2+y.^2);  c = x.*y-z;
%        figure
%        subplot(221); h = plotcol(x);
%        set(h, 'LineWidth',2); title('plotcol(..) for input ''x'' ');colorbar
%        subplot(222); plotcol(x,y,'o');title('plotcol(..) for input ''(x,y)''');colorbar 
%        subplot(223); h = plotcol(x,y,z);
%        set(h, 'LineWidth',2); title('plotcol(..) for input ''(x,y,z)''');colorbar
%        subplot(224); plotcol(x,y,z,c,'d');  title('plotcol(..) for input ''(x,y,z,c)''');colorbar

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Nico SNEEUW (NS), IAPG, Uni Munich
%    Markus ANTONI (MA), GI, Uni Stuttgart
% -------------------------------------------------------------------------
% revision history:
%    2021-05-04: MA, unified help text in VISBUNDLE
%    xxxx-xx-xx: NS, - Input arguments are transformed into column vectors. Column vector, 
%                      row vector and matrices are allowed now. (not always sensible)
%                    - 3d vs. 2d handling changed.
%   xxxx-xx-xx: NS,   V5-update: additional plot symbols (and marker symbol fill-in)
%   1999-11-xx, NS: separate between linestyle and marker after mesh command
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% -------------------------------------------------------------------------


if nargin == 0; error('Requires at least one input'); end
def_linestyle = '-';			% default line style
do3d = 0;					% default 2D view

% Determine which inputs were given:
if nargin == 1;                 		% 2D: X given.

  y = x(:)*[1 1];
  z = zeros(size(y));
  x = (1:length(x))'*[1 1];
  c = y;
  linestyle = def_linestyle;

elseif nargin == 2;  

  x = x(:)*[1 1];
  z = zeros(size(x));
  if ischar(y)                   		% 2D: X and LINESTYLE given.
    linestyle = y;
    y = x;
    x = (1:length(x))'*[1 1];
  else                          		% 2D: X and Y given.
    y = y(:)*[1 1];
    linestyle = def_linestyle;
  end
  c = y;

elseif nargin == 3;

  x = x(:)*[1 1];
  y = y(:)*[1 1];
  if ~ischar(z)                  		% 3D: X, Y, and Z given
    z = z(:)*[1 1];
    c = z;
    linestyle = def_linestyle;
    do3d = 1;
  else                          		% 2D: X, Y, and LINESTYLE given
    linestyle = z;
    z = zeros(size(x));
    c = y;
  end

elseif nargin == 4 

  x = x(:)*[1 1];
  y = y(:)*[1 1];
  z = z(:)*[1 1];
  if ischar(c)                   		% 2D or 3D: X, Y, Z=C, and LINESTYLE
    linestyle = c;
    c = z;
    if isempty(findstr(linestyle,'.ox+*sdv^<>ph')), do3d = 1;  end
    						% so lines yield 3D, markers 2D
  else						% 3D: with X, Y, Z, C
    linestyle = def_linestyle;
    c = c(:)*[1 1];
    do3d = 1;
  end

elseif nargin == 5				% 3D: Everything given.

  x = x(:)*[1 1];
  y = y(:)*[1 1];
  z = z(:)*[1 1];
  c = c(:)*[1 1];
  do3d = 1;

end

h = mesh(x,y,z,c);
if isempty(findstr(linestyle,'.ox+*sdv^<>ph'))
   set(h,'LineStyle',linestyle)
else
   set(h,'LineStyle','none')
   set(h,'Marker',linestyle)
   set(h,'MarkerFaceColor','flat')	% fill-in markersymbols
end
if ~do3d, view(2), end
if nargout == 1
  handle = h;
end

