function handle = plotcolm(X,Y,H,c,linestyle);
% handle = plotcolm(X,Y,H,c,linestyle);
% 
% PLOTCOLM visualizes observation locations within a map – with 2D or 3D coordinates – 
% and the observation values, which define then the colors.  The color
% scaling is determined by the values of C or by the current setting of
% CAXIS.  The scaled color values are used as indices into the current
% COLORMAP. A projection must be defined beforehand, e.g. with AXESM
%
% If C is not given, it is assigned to H.
%
% IN:
% X ............. x-coordinates of the observations in degree              [deg][NxM]
% Y ............. y-coordinates  of the observations in degre              [deg][NxM]
% H: ............ altitudes of the observations (optional)                      [NxM]
% c ............. value of the observations                                     [NxM]
% linestyle: .... defining the line/marker in MATLAB notation ('.ox+*sdv^<>ph')
% 
% OUT:
% handle:........ handle of the graphics for further improvement
%
% See also:  PLOTCOL
%
% sub-routines:
%    *   mfwdtran
%
% REMARKS:
%    *   The function requires the mapping toolbox of MATLAB
% 
% EXAMPLE:
%        t       = 0:.2:30;
%        phi     = -10 +t.*cos(t); 
%        lam  =  20 + t.*sin(t); 
%        obs     = sqrt(lambda.^2+phi.^2); 
%        alt     = randn(size(t));
%        getf = figure
%        load coastlines
%        axesm('eqaconic','Frame','on','Grid','on','MapLatLimit',[-40,20],'MapLonLimit',[-10,50])
%        plotcolm(phi,lambda,obs,'o'); hold on; 
%        plotm(coastlat, coastlon)
%        gridm('Grid','on','GColor',[0 0 0],'FontColor',[0 0 0])
%        geoshow('landareas.shp', 'FaceColor', [0.15 0.5 0.15])
%        framem;  tightmap; axis off

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Matthias WEIGELT (MW), GI, Uni Stuttgart
%    Markus ANTONI (MA), GI, Uni Stuttgart
% -------------------------------------------------------------------------
% revision history:
%    2021-05-04: MA, unified help text in VISBUNDLE, minimum 3 input arguments
%    ????-??-??: MW: modification of plotcol
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% -------------------------------------------------------------------------

if ~exist('mfwdtran','file'), error('''PLOTCOLM'' requires MATLAB and its mapping toolbox'); end

    
if nargin < 3; error('''PLOTCOLM'' requires at least 3 input argument'); end
def_linestyle = '-';			% default line style
do3d = 0;					        % default 2D view

% Determine which inputs are given:
if nargin == 3;

  X = X(:)*[1 1];
  Y = Y(:)*[1 1];
  if ~ischar(H)                  		% 3D: X, Y, and H=C given
    c = H(:)*[1 1];
    c = 0*H;
    linestyle = def_linestyle;
    do3d = 1;
    [X,Y,H] = mfwdtran(X,Y,H,'none');
  else                          		% 2D: X, Y, and LINESTYLE given
     error('''PLOTCOLM'' requires 3 numerical input arguments')
  end

elseif nargin == 4 

  X = X(:)*[1 1];
  Y = Y(:)*[1 1];
  H = H(:)*[1 1];
  [X,Y,H] = mfwdtran(X,Y,H,'none');
  if ischar(c)                   		% 2D X, Y, H=C, and LINESTYLE
    linestyle = c;
    c = H;
    if isempty(findstr(linestyle,'.ox+*sdv^<>ph')), do3d = 1;  end
    						% so lines yield 3D, markers 2D
  else						% 3D: with X, Y, H, C
    linestyle = def_linestyle;
    c    = c(:)*[1 1];
    do3d = 1;
  end

elseif nargin == 5				% 3D: Everything given.

  X = X(:)*[1 1];
  Y = Y(:)*[1 1];
  H = H(:)*[1 1];
  c = c(:)*[1 1];
  do3d = 1;
  [X,Y,H] = mfwdtran(X,Y,H,'none');

end

h = mesh(X,Y,H,c);
if isempty(findstr(linestyle,'.ox+*sdv^<>ph'))
   set(h,'LineStyle',linestyle)
else
   set(h,'LineStyle','none')
   set(h,'Marker',linestyle)
   set(h,'MarkerFaceColor','flat')	% fill-in markersymbols
end
if ~do3d, view(2), end
if nargout == 1
  handle = h;
end

