function handle = plotmulti(time, data,legendcell, ylimits, yticks, xticks,titleStr,color)
% handle = plotmulti(time, data,legendcell, ylimits, yticks, xticks,titleStr,color)
%
% PLOTMULTI visualizes several one-dimensional data sets within the same time
% interval in one figure and with repeated y-axes. A possible application is the 
% comparison of several instruments in one location. 
%
% For readablitity, only 12 data sets are allowed.
%
% IN:
% time: ..........argument/time                                                 [Nx1]
% data:.........  data set/observations                                         [NxM]
% legendcell ...  legend per data set as a cell                           {cell}[Mx1]
% ylimits:......  maximum/minimum of y-axis (equal for all sets)                [2x1]
% yticks:.......  labels of y-axis                                              [nx1]
% xticks:.......  labels of x-axis                                              [mx1]
% titleStr:.....  title of the complete figure                             ['string']
% color:........  set the color, e.g. color = {'r','b'} for red/blue signals   [cell]
%
% OUT:
% handle:........ handle of the graphics for further improvement
%
% See also: GRAYPATCH
%
% sub-routines:
%    *   --
%
% REMARKS:
%    *   The function uses the command SUBPLOT already. The application of 
%        PLOTMULTI within another subplot might cause unexpected behaviour.
%    *   If the vector COLOR has less entries, than the DATA matrix, then  
%        color will be repeat
%
% EXAMPLE:
%        t = 1:.1:10; y = bsxfun(@plus,2*sin(t),randn(4,numel(t))/10);
%        y(3,20:30) = 3.1; y(1,1:15) = NaN
%        legendcell = {'data 1','signal 2','inst. 3','set 4'}
%        figure; plotmulti(t, y ,legendcell);
%        figure; plotmulti(t,y,legendcell,[-3,3],[-2:.5:2],[0:1:10],'4 data sets',{'k','r','b'});

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Markus ANTONI (MA), GI, Uni Stuttgart
% -------------------------------------------------------------------------
% revision history:
%    2021-05-04: MA, new order of input & legend per data set is required
%                    unified help text in VISBUNDLE
%    2018-01-29: MA, integration into VISBUNDLE
%    2018-01-24: MA, initial version
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% -------------------------------------------------------------------------

%% checking argument 1,2,3
narginchk(3,8)
if ~ismatrix(data), error('''data'' must be a matrix'); end 
if ~isvector(time), error('''time'' must be a vector'); end   
if ~iscell(legendcell)
    warning('order of arguments is changed now, please check help text')
    error('''legendcell'' must be a cell array'); 
end  

%% default settings for argument 4...8
if nargin < 6, warning('''PLOTMULTI'': arguments 4,5 and 6 will improve the Figure'); end
if nargin < 4, ylimits =  1.2*([min(data(:)), max(data(:))]);   end 
if nargin < 5, yticks  =  ([min(data(:)), max(data(:))]);   end 
if nargin < 6, xticks  =  time(1:20:end);   end 
if nargin < 7, titleStr = 'Multiple data sets';   end
if nargin < 8, color = {'k'};      end

if ~iscell(color)
    color = {'r','b','k','g','r:','b:','k:','g:'};
end
if ~ischar(titleStr)
    titleStr = 'Multiple data sets';
end

% dimension of time and data
time = time(:);
time_anz = numel(time);
[row,col] = size(data);
if ismember(time_anz, [row,col]) == 0
    error('data set must have the same length as ''time''')
end
if row ~= time_anz
    data = data';
    col = row;
end
if col > 12
    error('The figures will be too small. Please split the data set')
end
try 
    legendcell{col};
catch
    error('''legendcell'' has not enough entries')
end

%% checking limits and labels
switch numel(ylimits)
case 2
    % everything ok
case 1
    ylimits = [-ylimits,ylimits];
otherwise
    error('interval ''ylimits'' must be a vector [2x1]')
end
xlimits = [min(time),max(time)];
if isvector(xticks) == false && numel(xticks) == 1
    error('argument ''xticks'' must be a vector')
end
if isvector(yticks) == false
    error('argument ''yticks'' must be a vector')
end
xticks  = unique(xticks);
yticks  = unique(yticks);
xtickrm = xticks(2:end-1);

%% plot several overlapping subplots 
height = 0.9/(col+1);
iter = 1;
iterMax = length(color);
for ii = col:-1:1
    subplot('Position',[0.1,ii*height,0.8,height])
    handle(ii) = plot(time,data(:,ii),color{iter});
    ylim(ylimits)
    xlim(xlimits)
    set(gca,'YTick',yticks)
    
    % x-ticks for all, and for the lowest figure:
    if ii > 1
        set(gca,'XTick',xtickrm)
    else
        set(gca,'XTick',xticks)
    end
    if ii == col
        title(titleStr) 
    end
    
    % try the given legends ...
    try 
        legend(legendcell{ii} )
    catch
    end
  
    % gray patches for every 2nd subplot
    if rem(ii,2) == 1
        set(gca,'Color',[0.94 0.94 0.94])
    end
    
    % if less colors then data are given, the color repeats:
    iter = iter+1;
    if iter > iterMax
        iter = 1;
    end
end
