function [xx,yy,zz] = sphere2(m,n)
%SPHERE2 Generate sphere.
%	[X,Y,Z] = SPHERE2(M,N) generates three (m+1)-by-(n+1)
%	matrices so that SURF(X,Y,Z) produces a unit sphere.
%
%	[X,Y,Z] = SPHERE2 uses M = 18, N=36, a 10 degree equi-angular grid.
%
%	SPHERE2(N) and just SPHERE2 graph the sphere as a SURFACE
%	and do not return anything.
%
% To a large extent, this function is the same as the matlab sphere function.
% The main difference is that the size of the mesh, specified by sphere2
% is specified both in latitude and in longitude direction.
%
% IN:
% m: ............ elements in latitude direction                                [1x1]
% m: ............ elements in longitude direction                               [1x1]
%
% OUT:
% X: ............ x-coordinate of the generated sphere                          [NxM]
% Y: ............ y-coordinate of the generated sphere                          [NxM]
% Z: ............ z-coordinate of the generated sphere                          [NxM]
%  
% See also: SPHERE (this function is modelled after sphere.m)
%
% sub-routines:

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Markus ANTONI (MA), GI, Uni Stuttgart
%    Nico SNEEUW (NS), Munich
% -------------------------------------------------------------------------
% revision history:
%    2021-04-21: MA, unified help text in VISBUNDLE
%    ????-??-??: NS, initial version
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% ------------------------------------------------------------------------- 

if nargin == 0, m = 18; n= 36; end
if nargin == 1, n= 2*m; end


% theta  (colatitude) is a column vector.
% lambda (longitude) is a row vector.
theta  = (0:m)'/m*pi;
lambda = (0:n)/n*2*pi;
ct = cos(theta);
st = sin(theta);  st(1) = 0; st(m+1) = 0;
cl = cos(lambda); 
sl = sin(lambda); sl(1) = 0; sl(n+1) = 0;

x      = st * cl;
y      = st * sl;
z      = ct * ones(1,n+1);

if nargout == 0
   surf(x,y,z)
else
   xx = x; yy = y; zz = z;
end
