function hh = sphrwarp(a,option,radfac)

% SPHRWARP   Warps a matrix on a (distorted) unit sphere
%
% HOW h = sphrwarp(a,option,radfac)
%     h = sphrwarp(a,b,radfac)
%
% IN 
% a ............. matrix to be visualized
% option ........ style of visualization 
%                    1 - spherical surface (default),
%                    2 - spherical mesh,
%                    3 - shaded deformed spherical surface
%                    b - Matrix B is warped onto spherical surface, deformed by A. 
% radfac ........ radial distortion factor [-1 1] (default 0)
% OUT 
% h .............handle to surface object
%  
% See also: 
%
% sub-routines:
%    *   SPHERE2
% 
% % REMARKS:
%    *   The surface object appears small. Use MYZOOM or similar afterwards.
%    *   Matrix A is used for surf (option 1), whereas the extended matrix Axt is
%        used for mesh (option 2). This allows 'shading interp' under option 2.
%    *   sphrwarp(ones(n,m),a) gives sphere with specified #meshlines
%    *   h=sprhwarp(a,2,.5); shading interp, set(h,'EdgeColor','k')
%            may be useful for small matrices A.
%
% EXAMPLE 
%       Ynm = ylm(10,4); 
%       figure
%       sphrwarp(Ynm,3,.3);
%       figure %% gives Ilk-style spherical harmonic
%       sphrwarp(Ynm,3,.5);colormap([1 1 1; .5 .5 .5])     

% -------------------------------------------------------------------------
% project: visBundle 
% -------------------------------------------------------------------------
% authors:
%    Markus ANTONI (MA), GI, Uni Stuttgart
%    Nico SNEEUW (NS), Munich
% -------------------------------------------------------------------------
% revision history:
%    2021-05-04: MA, unified help text in VISBUNDLE
%    2013-02-12: MA, example with the new SHBundle
%    1997-08-12: NS  few changes in order to have proper colorbar settings
%    1997-08-08: NS, based on the older version sphereplot
% -------------------------------------------------------------------------
% license:
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the  Free  Software  Foundation; either version 3 of the License, or
%    (at your option) any later version.
%  
%    This  program is distributed in the hope that it will be useful, but 
%    WITHOUT   ANY   WARRANTY;  without  even  the  implied  warranty  of 
%    MERCHANTABILITY  or  FITNESS  FOR  A  PARTICULAR  PURPOSE.  See  the
%    GNU General Public License for more details.
%  
%    You  should  have  received a copy of the GNU General Public License
%    along with Octave; see the file COPYING.  
%    If not, see <http://www.gnu.org/licenses/>.
% ------------------------------------------------------------------------- 

% diagnostics and defaults
if nargin < 3, radfac = 0; end			% def: no deformation
if nargin < 2, option = 1; end			% def: surface plot
if length(option) > 1
   b = option;
   option = 4;
end
if ~any(option == (1:4)), error('Illegal OPTION'), end
if abs(radfac) > 1, error('RADFAC should be within [-1 1]'), end

% here we go
point = get(gcf,'Pointer');
set(gcf,'Pointer','watch')

% create spherical mesh
[m,n]   = size(a);
[x,y,z] = sphere2(m,n);

% Extend matrix A, and deform radius
if option == 2 || radfac ~= 0
   [az,el,r] = cart2sph(x,y,z);
   mid    = mean(mean(a));
   range  = max(max(a)) - min(min(a));
   axt    = a([1 1:m m],:);                     % matrix A extended
   axt    = axt(:,[1 1:n n]);
   axt    = conv2(axt,ones(2,2)/4,'valid');     % Axt now (m+1)x(n+1)
   radius = 1 + radfac*(axt - mid)/range;       % unit radius + deformation
   [x,y,z]= sph2cart(az,el,radius);
end

% do the graphics
if option == 1
   h = surf(x,y,z,a);
elseif option == 2
   h = mesh(x,y,z,axt);
elseif option == 3
   [az,el]=view;
   h = surfl(x,y,z,[az+135,-el]);
   if max(m,n) > 100
      shading flat
   else
      shading interp
   end
elseif option == 4
   newplot;					% because of low-level graphics
   h = surface(x,y,z,'FaceColor','texture','EdgeColor','k','CData',b);
   set(h,'UserData',[min(b(:)) max(b(:))])	% required for colorbar
end

% some post-processing
axis off
axis square
axis([-1 1 -1 1 -1 1])
set(gcf,'Pointer',point)

% do output argument
if nargout > 0, hh=h; end
